<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV089-22-g45d1515b:2021-09-02#

require_once("UniModul.php");

class UniAdapterModel extends Mage_Payment_Model_Method_Abstract
{
	protected $uniModulName = null; //predefinuje binder
	protected $uniModul = null;
	protected $subMethod = '';


	protected $_code = '';
	//protected $_shouldInactivateQuote = false;   //nefunguje pro zachovani kosiku
	protected $_isGateway               = true;
    protected $_isInitializeNeeded      = true;
    protected $_formBlockType = 'platiti/form';
	protected $_canCapture              = true;

	// pro získani konfigurace pro controller, pro callback je totiz nutne aby byl adapter cely inicializovany
	public function getConfigDataUniExternal($uniModulName = null, $subMethod = null) {
		if ($uniModulName != null) $this->uniModulName = $uniModulName;
		if ($subMethod != null) $this->subMethod = $subMethod;
		return $this->initUniModul();
	}

    public function initialize($paymentAction, $stateObject)
    {
    	BeginUniErr();
        $state = Mage_Sales_Model_Order::STATE_NEW;
        $stateObject->setState($state);
        $stateObject->setStatus(Mage::getSingleton('sales/order_config')->getStateDefaultStatus($state));
        $stateObject->setIsNotified(false);
    	EndUniErr();
    }

    // jinak se nevola initialize
    public function getConfigPaymentAction() {
    	BeginUniErr();
    	$paymentAction = $this->getConfigData('payment_action');
    	return EndUniErr(empty($paymentAction) ? true : $paymentAction);
    }

	protected function initUniModul() {
		if ($this->uniModul == null) {
			if (is_null($this->uniModulName)) return;
			$uniFact = new UniModulFactory();
			$configInfo = $uniFact->getConfigInfo($this->uniModulName, $this->getLanguage(), $this->subMethod);
			$configSetting = $this->getConfigDataUni($configInfo, $this->uniModulName, $this->subMethod);
			$this->uniModul = $uniFact->createUniModul($this->uniModulName, $configSetting, $this->subMethod);
			return $configSetting;
		}
	}


    public function getOrderPlaceRedirectUrl() {
    	BeginUniErr();
    	//mage::log('Called custom ' . __METHOD__);
		$session = Mage::getSingleton('checkout/session');
		$session->setUniAdapterSubmethod($this->subMethod);
        return EndUniErr(Mage::getUrl(strtolower($this->uniModulName) . 'binder/' . strtolower($this->uniModulName) . 'binder/redirect', array('_secure' => true)));
    }

    public function isAvailable($quote = null) {
    	BeginUniErr();
    	if (!parent::isAvailable($quote)) {
    		return EndUniErr(false);
    	}
        if ($quote == null) {  // nevim proc se to muze volat s null, ale budeme optimisti
    		return EndUniErr(true);
    	}

    	$orderToPayInfo = $this->getOrderToPayInfo($quote, $this->subMethod);

    	$this->initUniModul();
    	$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);

    	return EndUniErr($prePayGWInfo->isPossible);
    }

	protected function getConfigDataUni($configInfo, $uniModulName, $subMethod) {

		// doplnime failed stav, i kdyz ho modul nevyzaduje, protoze Magento vzdy vytvori order
		if (array_filter($configInfo->configFields, array("UniAdapterModel", "getConfigDataUni_filter")) === array()) {
			$configField = new ConfigField();
			$configField->name = 'orderStatusFailed';
			$configField->type = ConfigFieldType::$orderStatus;
			$configInfo->configFields[] = $configField;
		}

		$configData = array();
		$path = "payment/" . strtolower($uniModulName) . "binder/";
		foreach ($configInfo->configFields as $configField) {
			$configData[$configField->name] = Mage::getStoreConfig($path . $configField->name);
		}

		$uniModulConfig = new UniModulConfig();

		$connConfig = Mage::getConfig()->getResourceConnectionConfig('core_write');

		$uniModulConfig->mysql_server = (string)$connConfig->host;
		$uniModulConfig->mysql_dbname = (string)$connConfig->dbname;
		$uniModulConfig->mysql_login = (string)$connConfig->username;
		$uniModulConfig->mysql_password = (string)$connConfig->password;

		$dbtype = $connConfig->type;
		if ($dbtype != 'mysql' && $dbtype != 'mysqli' && $dbtype != 'pdo_mysql') user_error('Eshop pouziva jinou nez mysql databazi');

		$uniModulConfig->uniModulDirUrl = Mage::getModel('core/url')->getStore()->getBaseUrl(Mage_Core_Model_Store::URL_TYPE_WEB).'UniModul/';
		$uniModulConfig->funcGetCallbackUrl = array($this, 'getCallbackUrl');

		$uniModulConfig->adapterName = 'Magento';
		$cfgs = create_initialize_object('ConfigSetting', array('configData'=>$configData, 'uniModulConfig'=>$uniModulConfig));

		return $cfgs;

	}

	private static function getConfigDataUni_filter($e) {
		return $e->name=='orderStatusFailed';
	}


	protected function getOrderToPayInfo($quote, $subMethod) {
    	$orderToPayInfo = new OrderToPayInfo();

		$orderToPayInfo->subMethod = $subMethod;

    	//$order_id = $quote->getRealOrderId();
    	$billing  = $quote->getBillingAddress();

    	//$orderToPayInfo->shopPairingInfo = $order_id;
    	//$orderToPayInfo->shopOrderNumber = $order_id;

    	$orderToPayInfo->amount = $quote->getGrandTotal();
    	$orderToPayInfo->currency = $quote->getQuoteCurrencyCode();

    	$description = Mage::app()->getStore()->getFrontendName();

    	$customerData = new CustomerData();
		$customerData->first_name = ($billing->getFirstname() ? $billing->getFirstname() : $quote->getCustomerFirstname());
		$customerData->last_name = ($billing->getLastname() ? $billing->getLastname() : $quote->getCustomerLastname());
    	$customerData->email = ($billing->getEmail() ? $billing->getEmail() : $quote->getCustomerEmail());
    	$customerData->street = $billing->getStreet(-1);
    	$customerData->city = $billing->getCity();
    	$customerData->post_code = $billing->getPostcode();
    	$customerData->country = $billing->getCountryModel()->getIso3Code();
    	$customerData->phone = $billing->getTelephone();
    	$orderToPayInfo->customerData = $customerData;

    	// toto orderToPayInfo neni urceno k presmerovani na branu
		//$code = strtolower($this->uniModulName) . 'binder';
    	//$orderToPayInfo->notifyUrl = Mage::getUrl($code . '/' . $code . '/notify', array('transaction_id_XX' => $order_id));
    	//$orderToPayInfo->replyUrl = Mage::getUrl($code . '/' . $code . '/reply', array('transaction_id_XX' => $order_id));

    	$orderToPayInfo->language = $this->getLanguage();

    	$orderToPayInfo->currencyRates = $this->getCurrencyRates();

    	//$orderToPayInfo->uniAdapterData = $session->getId ();  //mozna bych tam mohl strcit celou session i s daty

    	return $orderToPayInfo;
    }

	public function getCurrencyRates() {
		$currencyRates = array();
		$store = Mage::app()->getStore(null);
		$cur = $store->getBaseCurrency();
		foreach($cur->getConfigAllowCurrencies() as $c) {
			$currencyRates[$c] = $cur->getAnyRate($c);
		}
		return $currencyRates;
	}

	public function getCallbackUrl($callbackName, $arguments) {
		$code = strtolower($this->uniModulName) . 'binder';
		return Mage::getUrl($code . '/' . $code . '/callback')."?_unimodul=".$this->uniModul->name."&_callbackName=".urlencode($callbackName)."&".http_build_query($arguments);;
	}

     function getMiniLogo() {
		return $this->uniModul->getMiniLogoSpan();
    }

    function getImage() {
		return $this->uniModul->getModulSubMethodLogoImage2($this->uniModul->subMethod);
    }



	public function getInstallmentHtml() {
    	BeginUniErr();
		$currentProduct = Mage::registry('current_product');
		if (!is_null($currentProduct) && parent::isAvailable(null)) {
			$store = Mage::app()->getStore(null);
			if ($currentProduct->getTypeId() == Mage_Catalog_Model_Product_Type::TYPE_BUNDLE) {
				$priceModel = $currentProduct->getPriceModel();
				list($_minimalPriceInclTax, $_maximalPriceInclTax) = $priceModel->getTotalPrices($currentProduct, null, true, false);
				$price = $_maximalPriceInclTax;
			} else {
				$price = $currentProduct->getFinalPrice();
			}
			if ($store->getCurrentCurrency() && $store->getBaseCurrency()) {
				$product_price = $store->getBaseCurrency()->convert($price, $store->getCurrentCurrency());
			} else {
				$product_price = $price;
			}
			$current_currency = $store->getCurrentCurrencyCode();

			$baseUrl = Mage::getModel('core/url')->getStore()->getBaseUrl(Mage_Core_Model_Store::URL_TYPE_WEB);
			$this->initUniModul();
			$embedHtml = $this->uniModul->ProductGetInstallmentEmbedHtml($baseUrl, $current_currency, $product_price);

			return EndUniErr($embedHtml);
		} else {
			return EndUniErr('');
		}
	}

	public function getSuccessHtml() {
    	BeginUniErr();
		$session = Mage::getSingleton('core/session');
		if ($session->getData('UniModulSuccessHtmlUniModul') == $this->uniModulName && !empty($session->getData('UniModulSuccessHtml'))) {
			$html = "<style>.uniModulSuccessHtml td { padding-right:1ex; }</style><div class='main container'><div class='uniModulSuccessHtml'><p>&nbsp;<p/>".$session->getData('UniModulSuccessHtml')."</div></div>";
			$session->setData('UniModulSuccessHtml', null);
			return EndUniErr($html);
		} else {
			return EndUniErr('');
		}
	}

}
